# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2022 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# -------------------------------------------------------------------------------

# File:	modules/Ncpserver.ycp
# Package:	Configuration of ncpserver
# Summary:	Ncpserver settings, input and output functions
# Authors:	Stephen T. Bunch <sbunch@novell.com>
#
# $Id: Ncpserver.ycp,v 1.1.2.27 2005/10/04 20:23:07 rhaslam Exp $
#
# Representation of the configuration of ncpserver.
# Input and output routines.
require "yast"

module Yast
  class NcpserverClass < Module
    def main
      textdomain "novell-ncpserver"

      Yast.import "Progress"
      Yast.import "Popup"
      Yast.import "Report"
      Yast.import "Summary"
      Yast.import "Mode"
      Yast.import "Stage"
      Yast.import "Edirectory"
      Yast.import "EdirUtils"
      Yast.import "Service"
      Yast.import "NovellUtils"
      Yast.import "NovellLDAP"
      Yast.import "NovellPatterns"
      Yast.import "NovellAddonInstall"
      Yast.import "Wizard"

      # Settings Map
      @auto_clone = false
      @Settings = {}
      @oldSettings = {}

      @xad_packages_installed = false
      @xadMode = false

      @xadTestMode = false

      @schemaLogPath = "/var/opt/novell/eDirectory/log/oes_schema.log"

      # Data was modified?
      @modified = false


      @proposal_valid = false

      @upgrade_from = ""

      # Write only, used during autoinstallation.
      # Don't run services and SuSEconfig, it's all done at one place.
      @write_only = false
      @globals_configured = false

      # Abort function
      # return boolean return true if abort
      @AbortFunction = fun_ref(method(:Modified), "boolean ()")
    end

    # Abort function
    # @return [Boolean] return true if abort
    def Abort
      return @AbortFunction.call == true if @AbortFunction != nil
      false
    end

    # Data was modified?
    # @return true if modified
    def Modified
      Builtins.y2debug("modified=%1", @modified)
      @modified
    end

    # Read all ncpserver settings
    # @return true on success
    def Read
      # Ncpserver read dialog caption
      caption = _("Initializing NCP Server Configuration")

      # TODO FIXME Set the right number of stages
      steps = 2

      sl = 500
      Builtins.sleep(sl)

      # TODO FIXME Names of real stages
      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/3
          _("Read the database"),
          _("Checking dependencies")
        ],
        [
          # Progress step 1/3
          _("Reading the database..."),
          _("Checking dependencies"),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      # read database
      return false if Abort()
      Progress.NextStage


      # Test for the upgrade scenario
      @upgrade_from = NovellUtils.UpgradeIsFrom
      if (Mode.update || Mode.autoupgrade) &&  @upgrade_from != ""
        @oldSettings = NovellUtils.GetOldSysconfigMap("ncpserver")
      end

      Builtins.y2milestone(
        "ncpserver:
        upgradeFrom = #{@upgrade_from}"
        )
      Builtins.y2milestone("ncpserver:oldSettings: %1", @oldSettings)


      # See if XAD is installed.
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(
        "novell-domain-services"
      )

      @xad_packages_installed = true if @xadTestMode

      Builtins.y2milestone(
        "ncpserver:xad_packages_installed:%1",
        @xad_packages_installed
      )
      if @xad_packages_installed
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
            ) == "yes"
          @xadMode = true
        end
      end
      Builtins.y2milestone("ncpserver:xadMode:%1", @xadMode)

      # First, get the default LDAP/eDir data
      progress_orig = Progress.set(false)
      NovellLDAP.Read
      Progress.set(progress_orig)

      service_configured = Convert.to_string(
        SCR.Read(path(".sysconfig.ncpserver.SERVICE_CONFIGURED"))
      )
      service_configured = Builtins.tolower(service_configured)
      if service_configured == "yes"
        if Mode.normal
          name = "NCP Server"
          if !Popup.YesNo(
              Builtins.sformat(
                _(
                  "It appears that %1 has already been configured.  If %2 has been uninstalled\nand is being reinstalled you may continue.  Continue configuration?"
                ),
                name,
                name
              )
            )
            return false
          end #return (any)false;
        end
      end
      Builtins.sleep(sl)

      # check dependencies
      return false if Abort()
      Progress.NextStage

      @globals_configured = true

      return false if Abort()
      Progress.NextStage
      Progress.Title(_("Finished"))
      Builtins.sleep(sl)

      return false if Abort()
      @modified = false
      true
    end

    # Check to see if NSS is installed and has been configured.
    # @return true if NSS is installed.
    def NssInstalled
      # See if the novell-nss rpm is installed.
      rval = Convert.to_integer(
        SCR.Execute(path(".target.bash"), "rpm -q novell-nss")
      )
      if rval == 0
        # See if NSS has been configured.
        nss_configured = false
        service_configured = Convert.to_string(
          SCR.Read(path(".sysconfig.nss.SERVICE_CONFIGURED"))
        )
        service_configured = Builtins.tolower(service_configured)
        nss_configured = true if service_configured == "yes"
        if nss_configured
          Builtins.y2milestone(
            "Ncpserver:NssInstalled:NSS has been installed and configured."
          )
          return true
        else
          Builtins.y2milestone(
            "Ncpserver:NssInstalled:NSS has not been configured."
          )
        end
      else
        Builtins.y2milestone(
          "Ncpserver:NssInstalled:The novell-nss rpm is not installed."
        )
      end
      false
    end

    # Check to see if we need to restart the ncp2nss daemon and restart if necessary.
    # @return true if ncp2nss is started successfully.
    def checkRestartNcp2Nss
      execute_cmd = "/opt/novell/ncpserv/sbin/ncp2nss"

      # First check that NSS is installed.
      if NssInstalled() == false
        Builtins.y2milestone(
          "Ncpserver:checkRestartNcp2Nss:NSS is not installed. Returning."
        )
        return false
      end

      # Check for the existence of the executable.
      if SCR.Read(path(".target.size"), execute_cmd) == -1
        Builtins.y2milestone(
          "Ncpserver:checkRestartNcp2Nss:%1 is not present. Returning.",
          execute_cmd
        )
        return false
      end

      status = Service.Restart("ncp2nss")
      Builtins.y2milestone("Nss:Restarted ncp2nss service : %1",status)

      true
    end


    # Write all ncpserver settings
    # @return true on success
    def Write
      Builtins.y2milestone("Ncpserver::Write started")
      Wizard.DisableAbortButton
      pattern = "novell-ncp"
      if NovellPatterns.IsPatternInstalled(pattern) == false
        Builtins.y2milestone(
          "The required pattern, %1, is not installed.",
          pattern
        )
        if Mode.normal
          Report.Error(
            Builtins.sformat(
              _(
                "Configuration aborted. Install the required %1 pattern and try again."
              ),
              pattern
            )
          )
        end

        return true
      end

      sl = 500


      abortInstall = Builtins.tolower(
        NovellUtils.ReadOption("ABORT_OES_INSTALL")
      )
      if abortInstall == "yes"
        Builtins.y2milestone(
          "Ncpserver:setting ABORT_OES_INSTALL is set to 'yes'."
        )
        return true
      end
      # See if XAD is installed.
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(
        "novell-domain-services"
      )

      @xad_packages_installed = true if @xadTestMode

      Builtins.y2milestone(
        "ncpserver:xad_packages_installed:%1",
        @xad_packages_installed
      )
      if @xad_packages_installed
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
            ) == "yes"
          @xadMode = true
        end
      end
      Builtins.y2milestone("ncpserver:xadMode:%1", @xadMode)


      if !Mode.update && !Mode.autoupgrade
        if !Ops.get(NovellAddonInstall.products_to_configure, "ncpserver", true)
          Builtins.y2milestone("NCP Server -- configure later")
          return true
        end

        # If Aquila did not install properly, then we don't want to proceed at this point.
        if @xadMode == true
          Builtins.y2milestone(
            "Ncpserver:testing to see if eDir configured properly. Xad mode: %1",
            @xadMode
          )
          Builtins.y2milestone(
            "Ncpserver:eDir service_configured (mem):%1",
            Ops.get_string(Edirectory.EdirSettings, "SERVICE_CONFIGURED", "")
          )
          Builtins.y2milestone(
            "Ncpserver:eDir service_configured (file):%1",
            Convert.to_string(
              SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))
            )
          )
          if Ops.get_string(Edirectory.EdirSettings, "SERVICE_CONFIGURED", "") != "yes" &&
              Convert.to_string(
                SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))
              ) != "yes"
            if EdirUtils.EdirConfigProblemPrompt("NCP Server")
              Builtins.y2milestone(
                "Ncpserver:aborting NCP write by user choice since eDirectory was not configured."
              )
              return true
            end
          end
        end

        # Ncpserver read dialog caption
        caption = _("Saving NCP Server Configuration")

        # TODO FIXME And set the right number of stages
        steps = 1

        Builtins.sleep(sl)

        # TODO FIXME Names of real stages
        # We do not set help text here, because it was set outside
        Progress.New(
          caption,
          " ",
          steps,
          [
            # Progress stage 1/1
            _("Write the settings")
          ],
          [
            # Progress step 1/1
            _("Writing the settings..."),
            # Progress finished
            _("Finished")
          ],
          ""
        )

        # write settings
        return false if Abort()
        Progress.NextStage

        SCR.Write(path(".sysconfig.ncpserver.SERVICE_CONFIGURED"), "no")

        upgradeStr = ""
        if Mode.update || Mode.autoupgrade
          upgradeStr = "UPGRADE='1' "
        end

        # modify our conf file to add file server name
        cmd = "grep 'n4u.nds.server-name' /etc/nds.conf"
        retMap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
        if Ops.get_integer(retMap, "exit", -1) == 0
          returnStr = Ops.get_string(retMap, "stdout", "")
          _begin = Builtins.search(returnStr, "=")
          lineToAppend = "NCP_FILE_SERVER_NAME " # added underscores for bugzilla 95072.
          lineToAppend = Ops.add(
            lineToAppend,
            Builtins.substring(returnStr, Ops.add(_begin, 1))
          )
          cmd = Ops.add(
            Ops.add("echo '", lineToAppend),
            "' >> /etc/opt/novell/ncpserv.conf"
          )
          SCR.Execute(path(".target.bash"), cmd)
        end

        # first, back up the config file
        cmd = "cp /etc/opt/novell/httpstkd.conf /etc/opt/novell/httpstkd.conf.org"
        Builtins.y2milestone("Ncpserver.Write:executing: %1", cmd)
        val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
        Builtins.y2milestone("Ncpserver:val for backup is: %1", val)

        # add our line to the config file
        cmd = "echo 'load /opt/novell/lib/libnrm2ncp.so' >> /etc/opt/novell/httpstkd.conf "
        Builtins.y2milestone("Ncpserver.Write:executing: %1", cmd)
        val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
        Builtins.y2milestone("Ncpserver:val for adding line is: %1", val)

        admin_dn = NovellUtils.Ldap2Nds(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        )
        admin_password = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_ADMIN_PASSWORD",
          ""
        )

        if @xadMode
          xad_tree_admin_context = Ops.get_string(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_CONTEXT",
            ""
          )
          Builtins.y2milestone(
            "Ncpserver:XAD_TREE_ADMIN_CONTEXT:%1",
            xad_tree_admin_context
          )
          if xad_tree_admin_context != nil && xad_tree_admin_context != ""
            xad_tree_admin_context = NovellUtils.Ldap2Nds(
              xad_tree_admin_context
            )
            admin_dn = xad_tree_admin_context
            admin_password = Ops.get_string(
              NovellLDAP.Settings,
              "XAD_TREE_ADMIN_PASSWORD",
              ""
            )
          end
        end
        Builtins.y2milestone("Ncpserver:admin:%1", admin_dn)

        ndssch_file = "/opt/novell/eDirectory/bin/ndssch"
        ncpserv_sch_file = " /opt/novell/ncpserv/bin/ncpserv.sch"
        cmd = Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(Ops.add(ndssch_file, " -F '"), @schemaLogPath),
                "' '"
              ),
              admin_dn
            ),
            "' "
          ),
          ncpserv_sch_file
        )
        if @xadTestMode
          Builtins.y2milestone(
            "xad:Ncpserver:executing: %1",
            Ops.add(
              Ops.add(
                Ops.add(Ops.add(upgradeStr, "ADM_PASSWD='"), admin_password),
                "' "
              ),
              cmd
            )
          )
        else
          Builtins.y2milestone("Ncpserver:executing: %1", cmd)
        end

        rc = Convert.to_integer(
          SCR.Execute(
            path(".target.bash"),
            Ops.add(
              Ops.add(
                Ops.add(Ops.add(upgradeStr, "ADM_PASSWD='"), admin_password),
                "' "
              ),
              cmd
            )
          )
        )
        if rc != 0
          Builtins.y2milestone(
            "Ncpserver:Failed (error=%1) to extend schema using file %2",
            rc,
            ncpserv_sch_file
          )
        end

        create_volume_exe = "/opt/novell/ncpserv/bin/createVol"
        cmd = Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(Ops.add(create_volume_exe, " -a '"), admin_dn),
                "' -v "
              ),
              "sys"
            ),
            " -m "
          ),
          "/usr/novell/sys"
        )
        rc = Convert.to_integer(
          SCR.Execute(
            path(".target.bash"),
            Ops.add(
              Ops.add(
                Ops.add(Ops.add(upgradeStr, "OES_DATA='"), admin_password),
                "' "
              ),
              cmd
            )
          )
        )
        if @xadTestMode
          Builtins.y2milestone(
            "xad:Ncpserver:executing: %1",
            Ops.add(
              Ops.add(
                Ops.add(Ops.add(upgradeStr, "OES_DATA='"), admin_password),
                "' "
              ),
              cmd
            )
          )
        else
          Builtins.y2milestone("Ncpserver:executing: %1", cmd)
        end

        Builtins.y2milestone("Ncpserver:return code: %1", rc)
        if !Mode.update && !Mode.autoupgrade
          Report.Error(_("Unable to create volume object")) if rc != 0
        end

        if Mode.normal || Mode.installation && Stage.normal
          Builtins.y2milestone(
            "Ncpserver:restarting eDirectory and novell-httpstkd."
          )
          Service.Restart("ndsd")

          Service.Stop("novell-httpstkd")
          Service.Start("novell-httpstkd")

          Builtins.y2milestone(
            "Ncpserver:checking status of ncp2nss and restarting if necessary."
          )
          # If NSS is installed and this is a post install, then we also need to start the ncp2nss daemon (if not already started).
          checkRestartNcp2Nss
        else
          Service.Stop("ndsd")
          Service.Start("ndsd")
        end
        Builtins.y2milestone(
          "Ncpserver:checking to see if novell-cifs is installed."
        )
        if NovellPatterns.IsPatternInstalled("novell-cifs")
          Builtins.y2milestone("Ncpserver:restarting novell-cifs.")
          Service.Stop("novell-cifs")
          Service.Start("novell-cifs")
        end

        NovellUtils.SetFirewallPort(524, :tcp, true)
        NovellUtils.SetFirewallPort(524, :udp, true)
      else
        Builtins.y2milestone("Ncpserver.write:skipping config for upgrade.")
      end

      SCR.Write(path(".sysconfig.ncpserver.SERVICE_CONFIGURED"), "yes")
      SCR.Write(path(".sysconfig.ncpserver"), nil) # flush the cache

      return false if Abort()
      # Progress finished
      Progress.NextStage
      Progress.Finish
      Builtins.sleep(sl)

      return false if Abort()
      true
    end

    # Get all ncpserver settings from the first parameter
    # (For use by autoinstallation.)
    # @param [Hash] settings The YCP structure to be imported.
    # @return [Boolean] True on success
    def Import(settings)
      settings = deep_copy(settings)
      if settings == {}
        @globals_configured = false
        Ops.set(NovellAddonInstall.products_to_configure, "ncpserver", false)
        return true
      end
      @globals_configured = true

      true
    end

    # Dump the ncpserver settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def Export
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("novell-nss") == false

      if @auto_clone
        Ops.set(NovellAddonInstall.products_to_configure, "ncpserver", true)
      end
      if Ops.get(NovellAddonInstall.products_to_configure, "ncpserver", true)
        Ops.set(settings, "configure_now", true)
      end
      deep_copy(settings)
    end

    # Create a textual summary and a list of unconfigured cards
    # @return summary of the current configuration
    def Summary
      summary = ""

      if !@globals_configured
        nc = Summary.NotConfigured
        summary = Summary.AddLine(summary, nc)
      else
        summary = Summary.OpenList(summary)

        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Admin name with context: %1"),
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
          )
        )

        summary = Summary.CloseList(summary)
      end
      [summary, []]
    end

    # Create an overview table with all configured cards
    # @return table items
    def Overview
      # TODO FIXME: your code here...
      []
    end

    # Return packages needed to be installed and removed during
    # Autoinstallation to insure module has all needed software
    # installed.
    # @return [Hash] with 2 lists.
    def AutoPackages
      # TODO FIXME: your code here...
      { "install" => [], "remove" => [] }
    end

    publish :variable => :auto_clone, :type => "boolean"
    publish :variable => :Settings, :type => "map"
    publish :variable => :oldSettings, :type => "map"
    publish :variable => :xad_packages_installed, :type => "boolean"
    publish :variable => :xadMode, :type => "boolean"
    publish :variable => :xadTestMode, :type => "boolean"
    publish :variable => :schemaLogPath, :type => "string"
    publish :function => :Modified, :type => "boolean ()"
    publish :variable => :modified, :type => "boolean"
    publish :variable => :proposal_valid, :type => "boolean"
    publish :variable => :write_only, :type => "boolean"
    publish :variable => :globals_configured, :type => "boolean"
    publish :variable => :AbortFunction, :type => "boolean ()"
    publish :function => :Abort, :type => "boolean ()"
    publish :function => :Read, :type => "boolean ()"
    publish :function => :NssInstalled, :type => "boolean ()"
    publish :function => :checkRestartNcp2Nss, :type => "boolean ()"
    publish :function => :Write, :type => "boolean ()"
    publish :function => :Import, :type => "boolean (map)"
    publish :function => :Export, :type => "map ()"
    publish :function => :Summary, :type => "list ()"
    publish :function => :Overview, :type => "list ()"
    publish :function => :AutoPackages, :type => "map ()"
  end

  Ncpserver = NcpserverClass.new
  Ncpserver.main
end
